@@ -23,21 +23,17 @@ except (ImportError, AttributeError):
     IS_CPYTHON = True
     IS_PYPY = False
 
+from io import open as io_open
 try:
     from StringIO import StringIO
 except ImportError:
-    from io import StringIO
+    from io import StringIO  # doesn't accept 'str' in Py2
 
 try:
     import cPickle as pickle
 except ImportError:
     import pickle
 
-try:
-    from io import open as io_open
-except ImportError:
-    from codecs import open as io_open
-
 try:
     import threading
 except ImportError: # No threads, no problems
@@ -98,13 +94,15 @@ def get_distutils_distro(_cache=[]):
 
 
 EXT_DEP_MODULES = {
-    'tag:numpy' : 'numpy',
-    'tag:pstats': 'pstats',
-    'tag:posix' : 'posix',
-    'tag:array' : 'array',
-    'tag:coverage': 'coverage',
-    'tag:ipython': 'IPython',
-    'tag:jedi': 'jedi',
+    'tag:numpy':    'numpy',
+    'tag:asyncio':  'asyncio',
+    'tag:pstats':   'pstats',
+    'tag:posix':    'posix',
+    'tag:array':    'array',
+    'tag:coverage': 'Cython.Coverage',
+    'Coverage':     'Cython.Coverage',
+    'tag:ipython':  'IPython',
+    'tag:jedi':     'jedi',
 }
 
 def patch_inspect_isfunction():
@@ -270,6 +268,10 @@ def get_openmp_compiler_flags(language):
     if not gcc_version:
         return None # not gcc - FIXME: do something about other compilers
 
+    # gcc defines "__int128_t", assume that at least all 64 bit architectures have it
+    global COMPILER_HAS_INT128
+    COMPILER_HAS_INT128 = getattr(sys, 'maxsize', getattr(sys, 'maxint', 0)) > 2**60
+
     compiler_version = gcc_version.group(1)
     if compiler_version and compiler_version.split('.') >= ['4', '2']:
         return '-fopenmp', '-fopenmp'
@@ -279,6 +281,8 @@ try:
 except locale.Error:
     pass
 
+COMPILER = None
+COMPILER_HAS_INT128 = False
 OPENMP_C_COMPILER_FLAGS = get_openmp_compiler_flags('c')
 OPENMP_CPP_COMPILER_FLAGS = get_openmp_compiler_flags('cpp')
 
@@ -321,12 +325,14 @@ VER_DEP_MODULES = {
     (3,1): (_is_py3_before_32, lambda x: x in ['run.pyclass_special_methods',
                                                ]),
     (3,3) : (operator.lt, lambda x: x in ['build.package_compilation',
+                                          'run.yield_from_py33',
                                           ]),
-    (3,4,0,'beta',3) : (operator.le, lambda x: x in ['run.py34_signature',
-                                          ]),
+    (3,4): (operator.lt, lambda x: x in ['run.py34_signature',
+                                         ]),
+    (3,5): (operator.lt, lambda x: x in ['run.py35_pep492_interop',
+                                         ]),
 }
 
-COMPILER = None
 INCLUDE_DIRS = [ d for d in os.getenv('INCLUDE', '').split(os.pathsep) if d ]
 CFLAGS = os.getenv('CFLAGS', '').split()
 CCACHE = os.getenv('CYTHON_RUNTESTS_CCACHE', '').split()
@@ -549,7 +555,7 @@ class TestBuilder(object):
             for test in self.build_tests(test_class, path, workdir,
                                          module, mode == 'error', tags):
                 suite.addTest(test)
-            if mode == 'run' and ext == '.py' and not self.cython_only:
+            if mode == 'run' and ext == '.py' and not self.cython_only and not filename.startswith('test_'):
                 # additionally test file in real Python
                 suite.addTest(PureDoctestTestCase(module, os.path.join(path, filename)))
 
@@ -635,12 +641,11 @@ class CythonCompileTestCase(unittest.TestCase):
 
     def setUp(self):
         from Cython.Compiler import Options
-        self._saved_options = [ (name, getattr(Options, name))
-                                for name in ('warning_errors',
-                                             'clear_to_none',
-                                             'error_on_unknown_names',
-                                             'error_on_uninitialized') ]
-        self._saved_default_directives = Options.directive_defaults.items()
+        self._saved_options = [
+            (name, getattr(Options, name))
+            for name in ('warning_errors', 'clear_to_none', 'error_on_unknown_names', 'error_on_uninitialized')
+        ]
+        self._saved_default_directives = list(Options.directive_defaults.items())
         Options.warning_errors = self.warning_errors
         if sys.version_info >= (3, 4):
             Options.directive_defaults['autotestdict'] = False
@@ -933,14 +938,17 @@ class CythonCompileTestCase(unittest.TestCase):
         so_path = None
         if not self.cython_only:
             from Cython.Utils import captured_fd, print_bytes
+            from distutils.errors import CompileError, LinkError
             show_output = True
             get_stderr = get_stdout = None
             try:
                 with captured_fd(1) as get_stdout:
                     with captured_fd(2) as get_stderr:
                         so_path = self.run_distutils(test_directory, module, workdir, incdir)
-            except Exception:
-                if 'cerror' in self.tags['tag'] and get_stderr and get_stderr():
+            except Exception as exc:
+                if ('cerror' in self.tags['tag'] and
+                    ((get_stderr and get_stderr()) or
+                     isinstance(exc, (CompileError, LinkError)))):
                     show_output = False  # expected C compiler failure
                 else:
                     raise
@@ -1236,7 +1244,15 @@ class CythonPyregrTestCase(CythonRunTestCase):
                     module = import_ext(self.module, ext_so_path)
                     sys.stdout.flush() # helps in case of crashes
                     if hasattr(module, 'test_main'):
-                        module.test_main()
+                        # help 'doctest.DocFileTest' find the module path through frame inspection
+                        fake_caller_module_globals = {
+                            'module': module,
+                            '__name__': module.__name__,
+                        }
+                        call_tests = eval(
+                            'lambda: module.test_main()',
+                            fake_caller_module_globals, fake_caller_module_globals)
+                        call_tests()
                         sys.stdout.flush() # helps in case of crashes
                 except (unittest.SkipTest, support.ResourceDenied):
                     result.addSkip(self, 'ok')
@@ -1293,7 +1309,7 @@ def collect_doctests(path, module_prefix, suite, selectors, exclude_selectors):
     def package_matches(dirname):
         if dirname == 'Debugger' and not include_debugger:
             return False
-        return dirname not in ("Mac", "Distutils", "Plex")
+        return dirname not in ("Mac", "Distutils", "Plex", "Tempita")
     def file_matches(filename):
         filename, ext = os.path.splitext(filename)
         blacklist = ['libcython', 'libpython', 'test_libcython_in_gdb',
@@ -1555,9 +1571,8 @@ def refactor_for_py3(distdir, cy3_dir):
     # need to convert Cython sources first
     import lib2to3.refactor
     from distutils.util import copydir_run_2to3
-    fixers = [ fix for fix in lib2to3.refactor.get_fixers_from_package("lib2to3.fixes")
-               if fix.split('fix_')[-1] not in ('next',)
-               ]
+    with open('2to3-fixers.txt') as f:
+        fixers = [line.strip() for line in f if line.strip()]
     if not os.path.exists(cy3_dir):
         os.makedirs(cy3_dir)
     import distutils.log as dlog
@@ -1691,7 +1706,7 @@ def main():
                       action="store_true", default=False,
                       help="only compile pyx to c, do not run C compiler or run the tests")
     parser.add_option("--no-refnanny", dest="with_refnanny",
-                      action="store_false", default=not IS_PYPY,
+                      action="store_false", default=True,
                       help="do not regression test reference counting")
     parser.add_option("--no-fork", dest="fork",
                       action="store_false", default=True,
@@ -1702,7 +1717,7 @@ def main():
     parser.add_option("-x", "--exclude", dest="exclude",
                       action="append", metavar="PATTERN",
                       help="exclude tests matching the PATTERN")
-    parser.add_option("--shard_count", dest="shard_count", metavar="N",
+    parser.add_option("-j", "--shard_count", dest="shard_count", metavar="N",
                       type=int, default=1,
                       help="shard this run into several parallel runs")
     parser.add_option("--shard_num", dest="shard_num", metavar="K",
@@ -1756,10 +1771,9 @@ def main():
 
     WORKDIR = os.path.abspath(options.work_dir)
 
-    if sys.version_info[0] >= 3:
-        options.doctests = False
-        if options.with_cython:
-            sys.path.insert(0, options.cython_dir)
+    if options.with_cython and sys.version_info[0] >= 3:
+        sys.path.insert(0, options.cython_dir)
+        if sys.version_info[:2] == (3, 2):
             try:
                 # try if Cython is installed in a Py3 version
                 import Cython.Compiler.Main
@@ -1773,12 +1787,7 @@ def main():
                 # hasn't been refactored yet - do it now
                 global CY3_DIR
                 CY3_DIR = cy3_dir = os.path.join(WORKDIR, 'Cy3')
-                if sys.version_info >= (3,1):
-                    refactor_for_py3(DISTDIR, cy3_dir)
-                elif os.path.isdir(cy3_dir):
-                    sys.path.insert(0, cy3_dir)
-                else:
-                    options.with_cython = False
+                refactor_for_py3(DISTDIR, cy3_dir)
 
     if options.watermark:
         import Cython.Compiler.Version
@@ -1798,6 +1807,9 @@ def main():
                 coverage.erase()
                 coverage.start()
 
+    if options.xml_output_dir:
+        shutil.rmtree(options.xml_output_dir, ignore_errors=True)
+
     if WITH_CYTHON:
         global CompilationOptions, pyrex_default_options, cython_compile
         from Cython.Compiler.Main import \
@@ -1853,8 +1865,11 @@ def runtests(options, cmd_args, coverage=None):
     ROOTDIR = os.path.abspath(options.root_dir)
     WORKDIR = os.path.abspath(options.work_dir)
 
+    xml_output_dir = options.xml_output_dir
     if options.shard_num > -1:
         WORKDIR = os.path.join(WORKDIR, str(options.shard_num))
+        if xml_output_dir:
+            xml_output_dir = os.path.join(xml_output_dir, 'shard-%03d' % options.shard_num)
 
     # RUN ALL TESTS!
     UNITTEST_MODULE = "Cython"
@@ -1892,7 +1907,7 @@ def runtests(options, cmd_args, coverage=None):
         sys.path.insert(0, os.path.split(libpath)[0])
         CFLAGS.append("-DCYTHON_REFNANNY=1")
 
-    if options.xml_output_dir and options.fork:
+    if xml_output_dir and options.fork:
         # doesn't currently work together
         sys.stderr.write("Disabling forked testing to support XML test output\n")
         options.fork = False
@@ -1931,7 +1946,7 @@ def runtests(options, cmd_args, coverage=None):
 
     try:
         import jedi
-        if not ([0, 8, 1] <= list(map(int, re.findall('[0-9]+', jedi.__version__ or '0'))) < [0, 9]):
+        if not ([0, 9] <= list(map(int, re.findall('[0-9]+', jedi.__version__ or '0')))):
             raise ImportError
     except (ImportError, AttributeError, TypeError):
         exclude_selectors.append(RegExSelector('Jedi'))
@@ -1939,11 +1954,18 @@ def runtests(options, cmd_args, coverage=None):
     if options.exclude:
         exclude_selectors += [ string_selector(r) for r in options.exclude ]
 
+    if not COMPILER_HAS_INT128 or not IS_CPYTHON:
+        exclude_selectors += [RegExSelector('int128')]
+
     if options.shard_num > -1:
         exclude_selectors.append(ShardExcludeSelector(options.shard_num, options.shard_count))
 
     if not test_bugs:
-        exclude_selectors += [ FileListExcluder(os.path.join(ROOTDIR, "bugs.txt"), verbose=verbose_excludes) ]
+        exclude_selectors += [
+            FileListExcluder(os.path.join(ROOTDIR, bugs_file_name), verbose=verbose_excludes)
+            for bugs_file_name in ['bugs.txt'] + (['pypy_bugs.txt'] if IS_PYPY else []) +
+            (['windows_bugs.txt'] if sys.platform == 'win32' else [])
+        ]
 
     if sys.platform in ['win32', 'cygwin'] and sys.version_info < (2,6):
         exclude_selectors += [ lambda x: x == "run.specialfloat" ]
@@ -1997,6 +2019,8 @@ def runtests(options, cmd_args, coverage=None):
 
     if options.system_pyregr and languages:
         sys_pyregr_dir = os.path.join(sys.prefix, 'lib', 'python'+sys.version[:3], 'test')
+        if not os.path.isdir(sys_pyregr_dir):
+            sys_pyregr_dir = os.path.join(os.path.dirname(sys.executable), 'Lib', 'test')  # source build
         if os.path.isdir(sys_pyregr_dir):
             filetests = TestBuilder(ROOTDIR, WORKDIR, selectors, exclude_selectors,
                                     options.annotate_source, options.cleanup_workdir,
@@ -2008,9 +2032,12 @@ def runtests(options, cmd_args, coverage=None):
             sys.stderr.write("Including CPython regression tests in %s\n" % sys_pyregr_dir)
             test_suite.addTest(filetests.handle_directory(sys_pyregr_dir, 'pyregr'))
 
-    if options.xml_output_dir:
+    if xml_output_dir:
         from Cython.Tests.xmlrunner import XMLTestRunner
-        test_runner = XMLTestRunner(output=options.xml_output_dir,
+        if not os.path.exists(xml_output_dir):
+            try: os.makedirs(xml_output_dir)
+            except OSError: pass  # concurrency issue?
+        test_runner = XMLTestRunner(output=xml_output_dir,
                                     verbose=options.verbosity > 0)
     else:
         test_runner = unittest.TextTestRunner(verbosity=options.verbosity)
@@ -2066,8 +2093,6 @@ def runtests(options, cmd_args, coverage=None):
 if __name__ == '__main__':
     try:
         main()
-    except SystemExit: # <= Py2.4 ...
-        raise
     except Exception:
         traceback.print_exc()
         try:
@@ -2075,3 +2100,4 @@ if __name__ == '__main__':
         except PendingThreadsError:
             # normal program exit won't kill the threads, do it the hard way here
             flush_and_terminate(1)
+        sys.exit(1)
